% STK_OPTIM_EHVI_SMC optimizes the EHVI criteria using an SMC algorithm
%
% CALL: [x_opt, ehvi_opt, AUX] = stk_optim_ehvi_smc ...
%            (OPTIONS, DOM, MODEL, DATA, Z_REF)
%
%    optimizes the EHVI criterion using an SMC algorithm as in [1, 2].
%    Optimization done given a MODEL and some DATA in the domaine DOM, with
%    respect to a reference Z_REF. SMC algorithm need to be initialized.
%
% CALL: [...] = stk_optim_ehvi_smc (..., AUX)
%
%    continues a previous SMC run, using the data stored in the AUX structure.
%
% OPTIONS
%
%    .population_size   SMC sample size (default: 1000)
%    .updf              Target density: either PI (default) or EHVI
%    .display_2dim      Display some 2D-specific plots (default: false)
%
% REFERENCES
%
%   [1] Romain Benassi.  Nouvel algorithme d'optimisation bayesien
%       utilisant une approche Monte-Carlo sequentielle.  PhD Thesis,
%       Ecole Superieure d'Electricite (Supelec), 2013.
%
%   [2] Paul Feliot, Julien Bect, Emmanuel Vazquez.  A Bayesian approach
%       to constrained single- and multi-objective optimization.
%       Journal of Global Optimization, 67(1):97-133, 2017
%
%   [3] P. Del Moral, A. Doucet and A. Jasra.  Sequential Monte Carlo
%       samplers.  JRSS-B, 68(3):411-436, 2006.
%
%   [4] R. Douc and O. Cappé,  Comparison of resampling schemes for particle
%       filtering. In Proceedings of the 4th International Symposium on Image
%       and Signal Processing and Analysis (ISPA 2005), p. 64-69, 2005.
%
% See also stk_optim_ehvi, stk_optim_ehvi_grid, stk_optim_ehvi_ga,
%          stk_smc_initialize, stk_smc_reweight, stk_smc_resample, stk_smc_move

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function [x_opt, ehvi_opt, aux] = stk_optim_ehvi_smc ...
    (options, BOX, model, data, z_ref, aux)

% FIXME : implement a restart function

if nargin < 6
    aux = [];
end

% Default SMC sample size
if ~ isfield (options, 'population_size') || isempty (options.population_size)
    options.population_size = 1000;
end

% Check that batch_size is 1
if isfield (options, 'batch_size') && (options.batch_size > 1)
    stk_error ('options.batch_size > 1 not supported in this function')
end

% Default target density
if ~ isfield (options, 'updf') || isempty (options.updf)
    options.updf = 'pi';
end

% Default: no 2D display
if ~ isfield (options, 'display_2dim') || isempty (options.display_2dim)
    options.display_2dim = false;
end

dim = size (BOX, 2);
n = size (data, 1);

updf_ = @(x, data) compute_updf (BOX, model, data, z_ref, x, options.updf);
updf = @(x) updf_ (x, data);

if isempty (aux)
    [smc, sigma] = stk_smc_initialize (options.population_size, BOX, updf_, data);
    n_prec = 0;
else
    smc = aux.smc;
    sigma = aux.sigma;
    n_prec = n-1;
end

if options.display_2dim && dim == 2
    xg = double (stk_sampling_regulargrid (100^2, [], BOX));
    fig = figure(4);
    
    subplot(1,3,1); cla; hold on;
    pcolor_alias(xg, smc.updf(xg)); shading flat;
    plot(smc.particles(:,1), smc.particles(:,2), '.r');
    stk_title(['Density \Pi_{' num2str(n_prec) '}, scatter plot \Pi_{' num2str(n_prec) '}']);
    
    subplot(1,3,2); cla; hold on;
    pcolor_alias(xg, updf(xg)); shading flat;
    plot(smc.particles(:,1), smc.particles(:,2), '.r');
    stk_title(['Density \Pi_{' num2str(n) '}, scatter plot \Pi_{' num2str(n_prec) '}']);
end % if

smc = stk_smc_reweight (smc, updf);
smc = stk_smc_resample (smc);
[smc, sigma] = stk_smc_move (smc, sigma);

[x_opt, ehvi_opt] = stk_optim_ehvi_grid (smc.particles, model, data, z_ref);

if options.display_2dim && dim == 2
    figure(fig.Number);
    subplot(1,3,3); cla; hold on;
    pcolor_alias(xg, updf(xg)); shading flat;
    plot(smc.particles(:,1), smc.particles(:,2), '.r');
    stk_title(['Density \Pi_{' num2str(n) '}, scatter plot \Pi_{' num2str(n) '}']);
    drawnow; pause;
end % if

aux.smc = smc;
aux.sigma= sigma;

end % function


% Density definition
function updf = compute_updf (bounds, model, data, z_ref, x, updf)

switch lower (updf)
    
    case 'ehvi'
        updf = compute_ehvi (data, x, model, z_ref);
        
    case {'poi', 'pi'}
        updf = compute_poi (data, x, model, z_ref);
        
    otherwise
        error ('Unknown updf.');
        
end

updf(~ ismember (x, bounds)) = 0;

end % function


% Graphic function used to display
function pcolor_alias(X, Z)

n = sqrt(size(X, 1));
a = X(:,1);
b = X(:,2);

a = reshape(a, n, n);
b = reshape(b, n, n);
c = reshape(Z, n, n);

pcolor(a, b, c);

end % function
