% STK_PARETOFIND_CONSTRAINED sorts non-dominated rows, feasible but
% dominated rows and non-feasible rows in an array
%
% CALL: [PARETO_IND, NON_FEASIBLE_IND, FEASIBLE_DOMINATED_IND]
%           = stk_paretofind_constrained (Z, Z_MAX, WANTNONFEASIBLE)
%
%    returns the indices :
%
%      * PARETO_IND such that Z(PARETO_IND, :) contains all non-
%        dominated rows of Z.
%
%      * NON_FEASIBLE_IND such that Z(NON_FEASIBLE_IND, :) contains all non-
%        feasible rows of Z for the constraint Z <= Z_MAX
%
%      * FEASIBLE_DOMINATED_IND such that Z(FEASIBLE_DOMINATED_IND, :)
%        contains all feasible-dominated rows of Z.
%
% See stk_paretofind for definition of domination
%
% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup          <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin <g.maistre.bazin@gmail.com>
%              Lucain Pouget           <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function [idx_pareto, idx_nonfeas, idx_feasdom] ...
    = stk_paretofind_constrained (z, z_max, wantNonFeasible)

z = double (z);
z_max = double (z_max);

if(nargin < 3)
    wantNonFeasible = true;
end

[n, p] = size(z);
idx_pareto = [];
idx_nonfeas = [];
idx_feasdom = [];

if(wantNonFeasible)
    for i = 1:n
        for j = 1:p
            if( z(i, j) > z_max(j))
                z(i, :) = +Inf;
                idx_nonfeas = [idx_nonfeas ; i];
                break;
            end
        end
    end
    idx_pareto = stk_paretofind(z);
    if(length(idx_nonfeas) == n)
        idx_pareto = [];
    end
    idx_feasdom = setxor(1:n,[idx_pareto; idx_nonfeas]);
else
    paretoNonC = stk_paretofind(z);
    for i = 1:length(paretoNonC)
        isOk = true;
        for j = 1:p
            if( z(paretoNonC(i), j) > z_max(j))
                isOk = false;
            end
        end
        if(isOk)
            idx_pareto = [idx_pareto paretoNonC(i)];
        end
    end
end

end % function
