% STK_PLOT_BOO_2D plots (only when DIM = 2) bi-objective optimization
% during progress.
%
% CALL: [] = stk_plot_boo_2D (BOX, MODEL, X_OBS, Z_OBS, Z_REF, DHV_SEQ)
%
%    Plots the 2 estimated criterias, the EHVI criteria, the dominated
%    hypervolume progress and the Pareto Front in the objective-space.
%    Pareto front is computed based on the data (X_OBS, Z_OBS), considering
%    than only the points located below the refrence point Z_REF are feasible.
%
% See also stk_example_boo_01, stk_example_boo_02

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function [] = stk_plot_boo_2D (BOX, model, x_obs, z_obs, z_ref, DHV_seq)

DIM = 2;
if size (BOX, 2) ~= DIM
    errmsg = 'Only works for two-dimensional factor spaces.';
    stk_error (errmsg, 'IncorrectSize');
end

NT = 100;
xg = stk_sampling_regulargrid (NT^DIM, DIM, BOX);
xg.colnames = {'x1', 'x2'};

z_pred_1 = stk_predict (model(1), x_obs, z_obs(:, 1), xg);
z_pred_2 = stk_predict (model(2), x_obs, z_obs(:, 2), xg);

EHVI_val = stk_sampcrit_cehvi_eval ([z_pred_1.mean z_pred_2.mean], ...
    sqrt ([z_pred_1.var z_pred_2.var]), z_obs, z_ref);

[idx_pareto, idx_nonfeas, idx_feasdom] = stk_paretofind_constrained (z_obs, z_ref);
x_pareto  = x_obs(idx_pareto, :);
x_nonfeas = x_obs(idx_nonfeas, :);
x_feasdom = x_obs(idx_feasdom, :);

xg_min = min (double (xg));
xg_max = max (double (xg));
ax = [xg_min(1) xg_max(1) xg_min(2) xg_max(2)];

figure (501);  % FIXME: Do not use fixed figure number

subplot (2, 2, 1);  cla;  hold on;  axis (ax);
stk_plot2D_on_grid (xg, z_pred_1.mean, ...
    x_pareto, x_feasdom, x_nonfeas, x_obs(end,:));
stk_title ('posterior mean: \xi_1');

subplot (2, 2, 2);  cla;  hold on;  axis (ax);
stk_plot2D_on_grid (xg, z_pred_2.mean, ...
    x_pareto, x_feasdom, x_nonfeas, x_obs(end,:));
stk_title ('posterior mean: \xi_2');

EHVI_max = max (EHVI_val);
subplot (2, 2, 3);  cla;  hold on;  axis(ax);
stk_plot2D_on_grid (xg, log (EHVI_val + EHVI_max * 1e-7), ...
    x_pareto, x_feasdom, x_nonfeas, x_obs(end,:));
stk_title ('EHVI criterion');

subplot (2, 2, 4);
plot (DHV_seq,'-')
stk_title ('dominated HV');

figure (502);  % FIXME: Do not use fixed figure number
stk_plot_boo_pareto (z_obs, z_ref, 1);

end % function


function [] = stk_plot2D_on_grid ...
    (xg, zg, x_pareto, x_feasdom, x_nonfeas, x_last)

pcolor (xg, zg);  shading flat;

plot (x_feasdom(:, 1), x_feasdom(:, 2), 'r*');
plot (x_nonfeas(:, 1), x_nonfeas(:, 2), 'x', 'color', [1 0 0.5]);
plot (x_pareto(:, 1), x_pareto(:, 2) ,'.g','markersize',20);
plot (x_last(:, 1), x_last(:, 2), 'or');

end % function
