% STK_PLOT_BOO_2D_PROBLEM plots criterias and Pareto front for constrained
% bi-objective optimization problem in dimension 2.
%
% CALL: [] = stk_plot_boo_2D_problem (BOX, F1, F2, ZR)
%
%    plots the criterias F1 and F2 (function handles) with a pcolor in the
%    defined BOX.  Also plots the Pareto front, where only the points located
%    below the reference point ZR are considered feasible.
%
% IMPORTANT REMARK
%
%    This function does a large number of calls to the objective functions F1
%    and F2, which are assumed to be cheap to evaluate.
%
% See also stk_example_boo_01, stk_example_boo_02

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function stk_plot_boo_2D_problem (BOX, f1, f2, zr)

if size (BOX, 2) ~= 2
    errmsg = 'Only works for two-dimensional factor spaces.';
    stk_error (errmsg, 'IncorrectSize');
end

% Define a 2D-regular-grid
xg = stk_sampling_regulargrid (10000, 2, BOX);
xg.colnames = {'x1', 'x2'};

% Evalute F1 and F2 on the grid
zg = [feval(f1, xg) feval(f2, xg)];

% Determine Pareto front
idx_pareto = stk_paretofind_constrained (zg, zr);
xg_pareto  = xg(idx_pareto, :);
zg_pareto  = zg(idx_pareto, :);

% Display Pareto front
subplot (2, 1, 2);  hold on
plot (zg(:,1), zg(:,2), '.', 'Color', 'b');
plot (zg_pareto(:, 1), zg_pareto(:, 2), '.g', 'MarkerSize', 20);
axis auto;  stk_xlabel ('f1');  stk_ylabel ('f2');  stk_title ('objective space');
legend ('dominated points' , 'Pareto front');

% Display evaluations
subplot (2, 2, 1);  hold on
pcolor (xg, zg(:, 1));
plot (xg_pareto(:, 1), xg_pareto(:, 2), '.g', 'MarkerSize', 20);
stk_title ('first objective');

subplot (2, 2, 2); hold on
pcolor (xg, zg(:, 2));
plot (xg_pareto(:, 1), xg_pareto(:, 2), '.g', 'MarkerSize', 20);
stk_title ('second objective');

end % function
