% STK_PLOT_BOO_PARETO plots Pareto front for constrained bi-objective
% optimization problem
%
% CALL: stk_plot_boo_pareto (ZI, ZR)
%
%    plots the evaluation point ZI with different color code whether the point
%    is non-feasible (i.e., not dominated by ZR),  feasible but dominated (by
%    some other observation point) or feasible and not dominated.  Also plots
%    feasible area (i.e., the area below ZR) and last evaluation.
%
% CALL: stk_plot_boo_pa reto (ZI, ZR, BATCHSIZE)
%
%    also plots BATCHSIZE points as "last evaluated points".
%
% CALL: stk_plot_boo_pareto (ZI, ZR, BATCHSIZE, ZT)
%
%    also plots the true Pareto front ZT if it is known.
%
% See also stk_example_boo_01

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function stk_plot_boo_pareto (zi, zr, batchSize, zt)

if nargin < 3
    batchSize = 1;
end % if

cla; hold on;

bmin = [min(zi(:, 1)) min(zi(:, 2))];
bmax = [max([zi(:, 1); zr(1)]) max([zi(:, 2); zr(2)])];

delta = bmax - bmin;
bmin = bmin - 0.05 * delta;
bmax = bmax + 0.05 * delta;

% Determine Pareto from data
[idx_pareto, idx_nonfeas, idx_feasdom] = stk_paretofind_constrained (zi, zr);
zi_pareto  = zi(idx_pareto, :);
zi_nonfeas = zi(idx_nonfeas, :);
zi_feasdom = zi(idx_feasdom, :);

% Plot
h1 = fill([bmin(1) bmin(1) zr(1) zr(1)], [bmin(2) zr(2) zr(2) bmin(2)], [255 210 210]/255,  'edgecolor','none');
ppfp = getParetoFrontToPlot([zi_pareto(:,1) zi_pareto(:,2)], zr);
if ~ isempty (ppfp)
    plot(ppfp(:,1), ppfp(:,2),'g-','markersize',20);
end % if
h2 = plot(zi_feasdom(:,1), zi_feasdom(:,2), 'r*');
h3 = plot(zi_nonfeas(:,1), zi_nonfeas(:,2), 'x', 'color', [1 0 0.5]);
h4 = plot(zi_pareto(:,1), zi_pareto(:,2), '.g', 'markersize', 20);
h5 = plot(zi((end-batchSize+1):end, 1), zi((end-batchSize+1):end, 2), 'or');

if nargin > 3
    % zt may contain non-feasible points
    zt = zt(stk_paretofind_constrained (zt, zr, false), :);
    ppfpTheo = getParetoFrontToPlot (zt, zr);
    p7 = plot(ppfpTheo(:,1), ppfpTheo(:,2) ,'-b');
end % if

% Set labels and legend
axis ([bmin(1) bmax(1) bmin(2) bmax(2)]);
stk_xlabel('f1');
stk_ylabel('f2');
leg_h = [];
leg_txt = {};

[leg_h, leg_txt] = addToLegend (leg_h, leg_txt, h1, 'Integration area');

s = sprintf ('Feasible non-dominated points (%d)', size (zi_pareto, 1));
[leg_h, leg_txt] = addToLegend (leg_h, leg_txt, h4, s);

s = sprintf ('Feasible dominated points (%d)', size (zi_feasdom, 1));
[leg_h, leg_txt] = addToLegend (leg_h, leg_txt, h2, s);

s = sprintf ('Non-feasible points (%d)', size (zi_nonfeas, 1));
[leg_h, leg_txt] = addToLegend (leg_h, leg_txt, h3, s);

if batchSize == 1
    s = 'Last evaluation';
else
    s = sprintf ('Last batch evaluated (%d)', batchSize);
end
[leg_h, leg_txt] = addToLegend (leg_h, leg_txt, h5, s);

if exist('p7','var') == 1
    [leg_h, leg_txt] = addToLegend ...
        (leg_h, leg_txt, p7, 'True Pareto');
end % if

legend (leg_h, leg_txt);
stk_title ('objective space');

end % function


% Get stair points for Pareto
function res = getParetoFrontToPlot (pareto_pts, zr)

n = size(pareto_pts(:,1),1);
res = [];

if(n > 0)
    res = [res; [pareto_pts(1,1) zr(2)] ];
    for i = 1:(n-1)
        res = [ res;
            pareto_pts(i,:);
            [pareto_pts(i+1,1) pareto_pts(i,2)] ];
    end
    res = [res; pareto_pts(n,:); [zr(1) pareto_pts(n,2)]];
end

end % function


% Add new line in legend if p is plotted
function [legendPlots, legendTxt] = addToLegend(legendPlots, legendTxt, p, txt)

if(~isempty(p))
    legendPlots = [legendPlots, p];
    legendTxt = [legendTxt txt];
end

end % function
