% STK_SAMPCRIT_CEHVI_EVAL a computes constrained variant of EHVI
%
% CALL: EHVI = stk_sampcrit_cehvi_eval (ZP_MEAN, ZP_STD, ZI, ZRU, ZRL)
%
%    computes a contrained EHVI, where solutions are considered feasible
%    when they dominate the upper reference point ZRU.
%
% FIXME : constrained EHVI is not exactly the same as in the article [1] (???)
%
%
% REFERENCES
%
%   [1] P. Feliot and J. Bect and E. Vazquez, "A Bayesian approach to
%       constrained single- and multi-objective optimization", 2016
%
% See also stk_sampcrit_ehvi_eval, stk_optim_ehvi

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function EHVI = stk_sampcrit_cehvi_eval (zp_mean, zp_std, zi, zru, zrl)

zp_mean = double (zp_mean);
zp_std  = double (zp_std);
zi      = double (zi);

[n, nb_obj] = size (zp_mean);

if nargin < 4
    % zrl = zeros (1, nb_obj);  % Not implemented yet (see below)
    zru = ones (1, nb_obj);
else
    % zrl = double (zrl);  % Not implemented yet (see below)
    % if any (~ isfinite (zrl))
    %     error ('The components of zrl must be finite.')
    % end
    zru = double (zru);
    if any (~ isfinite (zru))
        error ('The components of zru must be finite.')
    end
end

hasFeasible = any (all (bsxfun (@le, zi, zru), 2));

if hasFeasible
    
    % FIXME: why can't simply call the original stk_sampcrit_cehvi_eval here ?
    
    % EHVI with respect to the reference
    EIr = stk_distrib_normal_ei (zru, zp_mean, zp_std, 1);  % m x p
    EHVI = prod (EIr, 2);                                  % m x 1
    
    % Compute signed decomposition wrt to the reference zr
    S = stk_dominatedhv (zi, zru, 1);
    
    if ~ isempty (S.sign)
        
        % Shift rectangle number to third dimension
        Rs = shiftdim (S.sign,  -2);
        Ra = shiftdim (S.xmin', -1);
        Rb = shiftdim (S.xmax', -1);
        
        EIa = stk_distrib_normal_ei (Ra, zp_mean, zp_std, 1);  % m x p x R
        EIb = stk_distrib_normal_ei (Rb, zp_mean, zp_std, 1);  % m x p x R
        
        EHVI = EHVI - sum (bsxfun (@times, Rs, prod (EIb - EIa, 2)), 3);
        
    end % if
    
else
    
    %%% JB, 2017/06/28:  I haven't checked this part of the code in detail, but
    %%% it looks rather suspicious...  I prefer to indicate temporarily that it
    %%% is not ready to be used.
    
    error ('Not implemented yet.');
    
    %     EHVI = zeros(n, 1);
    %     for i = 1:size(zp_mean,1)
    %         if all (zp_mean(i,:) < zru)  % FIXME:  WTF ?!?
    %             v = (c < zp_mean(i,:));
    %
    %             z_tmp_mean = diag(zp_mean(i,:)-zrl) + zrl * ones(nb_obj);
    %             z_tmp_mean = z_tmp_mean(v,:);
    %             z_tmp_std = diag(zp_std(i,:)-zrl) + zrl * ones(nb_obj);
    %             z_tmp_std = z_tmp_std(v,:);
    %
    %             S = stk_dominatedhv (z_tmp_mean, zru, 1);
    %
    %             if ~ isempty (S.sign)
    %                 xmin_std = S.xmin;
    %                 [a,b] = ismember(xmin_std,z_tmp_mean);
    %                 xmin_std(a) = z_tmp_std(b(a));
    %                 EIr = stk_distrib_normal_ei (S.xmax, S.xmin, xmin_std, 1);
    %                 EHVI(i) = sum(S.sign .* prod (EIr, 2) );
    %             end % if
    %         end % if
    %     end % for
    
end % if

end % function
