% STK_SMC_MOVE move the particles of an SMC sample
%
% CALL: SMC = stk_smc_move (SMC, SIGMA)
%
%    of the particles of the sample SMC using a Random Walk Metropolis-Hastings
%    (RWMH) algorithm with Gaussian increments.  The increments are independent
%    across dimensions, with standard deviations given by the vector SIGMA.
%
% FIXME: Update doc... (now uses an adaptive MCMC scheme based on [1])
%
% REFERENCE
%
%  [1] Julien Bect, Ling Li and Emmanuel Vazquez.  Bayesian Subset Simulation.
%      SIAM/ASA Journal on Uncertainty Quantificatio (to be published).
%      arXiv:1601.02557
%
% See also stk_optim_ehvi_smc, stk_smc_initialize,
%          stk_smc_reweight, stk_smc_resample

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Julien Bect  <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

function [smc, sigma] = stk_smc_move (smc, sigma)

[pop_size, dim] = size (smc.particles);

% Make sure sigma has the appropriate size
sigma = reshape (sigma, 1, dim);

% Same values as in the BSS paper
NB_STEPS = 10;
TARGET_RATE = 0.30;
DELTA_INIT = log (10);

% Make several steps of RW-MH
for t = 1:NB_STEPS
    
    % Draw Gaussian increments
    incr = bsxfun (@times, randn (pop_size, dim), sigma);
    
    % Compute proposed steps
    particles1 = smc.particles + incr;
    
    % Compute the corresponding un-normalized pdf
    updf1 = smc.updf_fun (particles1);
    
    % Compute Metropolis-Hasting ratios (symmetric proposal)
    a = min (1, updf1 ./ smc.updf_val);
    if any (isnan (a))
        error ('Some acceptance ratios are equal to NaN');
    end
    
    % Decide at random to accept or reject particles
    b = (rand (pop_size, 1) < a);
    
    % (UNCOMMENT TO:) Display acceptance rate
    % fprintf ('stk_smc_move [%d]: acceptance rate = %.1f%%\n', t, mean (b) * 100);
    
    % Update SMC structure
    smc.particles( b, :) = particles1(b, :);
    smc.updf_val( b, :) = updf1(b, :);
    
    % Adaptation
    direct = 2 * (mean (a) > TARGET_RATE) - 1;
    sigma = sigma * (exp (DELTA_INIT / t * direct));
    
end

end % function
