% STK_EXAMPLE_BOO01  A simple illustration of Bayesian bi-objective
% optimization
%
% Our goal here is to optimize the bi-objective problem
%
%    (x1,x2) |--> x1^2 + (x2-7)^2
%    (x1,x2) |--> stk_testfun_braninhoo(x1,x2)
%
% over [ [-5; 10] [0; 15] ].
%
% A Matern 5/2 prior with known parameters is used.
%
% Evaluations points are chosen for Niter = 10 iterations sequentially
% using the Expected Hypervolume Improvement (EHVI) criterion, starting
% from an initial design of N0 = 20 points.

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

%% Problem definition
clear

% Input space (hyper-rectangle object)
dom = stk_hrect ([ [-5; 10] [0; 15] ], {'x1', 'x2'});

f1 = @(x) x(:,1).^2+(x(:,2)-7).^2;    % Define first 1D function
f2 = @(x)(stk_testfun_braninhoo(x));  % Define second 1D function
z_ref = [200 200];                    % Define ref point in objective space

NIter = 50; % Number of points to be added adaptively


%% Initial design of experiments

% Size of the initial design
n0 = 10;

% Construction of the initial design
x0 = stk_sampling_maximinlhs (n0, [], dom);

% Values of the function on the initial design
z0 = stk_feval ({f1 f2}, x0);
z0.colnames = {'z1', 'z2'};


%% Display problem

figure (1);  stk_plot_boo_2D_problem (dom, f1, f2, z_ref);


%% Specification of the model (Gaussian process prior)

dim = size (dom, 2);  % Dimension of the input space

% Parameters for the Matern covariance function
% ("help stk_materncov52_iso" for more information)
model(1) = stk_model ('stk_materncov52_aniso', dim);
model(2) = stk_model ('stk_materncov52_aniso', dim);

model(1).param = [15 -5 -5];
model(2).param = [11 -3 -4];


%% Sequential design of experiments
%
% Here, evaluations points are chosen sequentially using the Expected
% Hypervolume Improvement (EHVI) criterion, starting from the initial
% design defined above.
%
% The algorithm stops when the maximum number of iterations is reached.

% Start with the initial design defined above
x = x0;
z = z0;

DHV_seq = zeros (NIter, 1);

optim_options = struct ();
optim_options.method = 'SRS';        % See help stk_optim_ehvi for details
optim_options.population_size = [];  % empty means: use the default

aux = [];
for i = 1:NIter
    
    % Optimisation of EHVI criterion
    [x_new, ehvi_opt, aux] = stk_optim_ehvi ...
        (optim_options, dom, model, horzcat (x, z), z_ref, aux);
    
    DHV_seq(i) = ehvi_opt;
    
    % Evaluation
    x_new = double (x_new);
    z_new = stk_feval ({f1 f2}, x_new);
    
    % Add new point
    x = vertcat (x, x_new);
    z = vertcat (z, z_new);
    
    % Extract non-dominated solutions
    idx_pareto = stk_paretofind_constrained (z, z_ref);
    x_pareto = x(idx_pareto, :);
    z_pareto = z(idx_pareto, :);
    
    % Compute dominated hypervolume
    DHV_seq(i) = stk_dominatedhv (z_pareto, z_ref);
    
    % Plot progress
    stk_plot_boo_2D (dom, model, x, z, z_ref, DHV_seq(1:i));  drawnow
    set (gcf, 'Name', sprintf ('n = %d + %d = %d', n0, i, n0 + i));
        
    pause
end


%#ok<*AGROW>
