% STK_EXAMPLE_BOO03  An illustration of Bayesian bi-objective
% optimization with DIM > 2. Example has two problems and parameters to
% play with. Possibility to choose the algortihm to optimize EHVI
% criterion and compute simulation by batch.
%
% Our goal here is to optimize a bi-objective problem. You can choose it :
%
%  Problem 1 :
%    (x1,x2,x3,x4,x5) |--> 4*x1^2+7*(x2-1).^2+x3^3+4*x4^2-(x5-5)^2;
%    (x1,x2,x3,x4,x5) |--> ((x1-3)^2+(x2-5)^2)*((x5-8)^2+(x4-5)^2)+x3;
%    over [ [0; 5] [0; 3] [0; 5] [0; 3] [0; 7] ].
%
%  Problem 2 :
%    Schaffer problem with DIM = 8. Please see reference for details [1]
%
% REFERENCES
%   [1] M. Emmerich and K. Giannakoglou and B. Naujoks "Single- and Multi-
%       objective Evolutionary Optimization Assisted by Gaussian Random
%       Field Metamodels"

% Copyright Notice
%
%    Copyright (C) 2017 CentraleSupelec
%
%    Authors:  Etienne Leloup           <etienne.leloup@free.fr>
%              Guillaume Maistre-Bazin  <g.maistre.bazin@gmail.com>
%              Lucain Pouget            <lucainp@gmail.com>
%              Julien Bect              <julien.bect@centralesupelec.fr>

% Copying Permission Statement
%
%    This file is part of STK-contrib-moo-1
%      (https://sourceforge.net/p/kriging/contrib-moo-1/).
%
%    STK-contrib-moo-1 is free software;  you can redistribute it  and/or
%    modify it  under the terms of the  GNU Lesser General Public License
%    as published by the Free Software Foundation;  either version 2.1 of
%    the License, or (at your option) any later version.
%
%    STK-contrib-moo-1 is distributed in the hope that it will be useful,
%    but  WITHOUT  ANY  WARRANTY;  without even  the implied  warranty of
%    MERCHANTABILITY or FITNESS  FOR A  PARTICULAR PURPOSE.  See the  GNU
%    Lesser General Public License for more details.
%
%    You should  have received  a copy  of the  GNU Lesser General Public
%    License along with STK-contrib-moo-1;  if not, see
%                                          <http://www.gnu.org/licenses/>.

%% Play with parameters to understand the cases

NIter = 50;           % Number of points to be added adaptively
NPause = Inf;         % Number of points to be added between each pause
NEstimParam = 10;     % Number of points to be added between each reestimation of the parameters
problem = 1;          % See script description
batchSize = 1;        % Size of the batch (for parallelisation)


%% Problem definition

switch problem
    
    case 1 % Polynomial, DIM = 5
        dim = 5;
        dom = stk_hrect ([[0; 5] [0; 3] [0; 5] [0; 3] [0; 7]], ...
            {               'x1'   'x2'   'x3'   'x4'  'x5'});
        f1 = @(x) 4*x(:,1).^2+7*(x(:,2)-1).^2+x(:,3).^3+4*x(:,4).^2-(x(:,5)-5).^2;
        f2 = @(x) ((x(:,1)-3).^2+(x(:,2)-5).^2).*((x(:,5)-8).^2+(x(:,4)-5).^2)+x(:,3);
        z_ref = [200 1000];
        % For this problem the exact solution is unknown:
        dhv_theo = 0;
        true_pareto_f1 = [];
        true_pareto_f2 = [];
        
    case 2 % Schaffer problem, DIM = 8
        dim = 8;
        dom = stk_hrect (repmat ([0; 10], 1, dim), ...
            {'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7', 'x8'});
        gamma = 1; cte = 1/dim^(gamma/2);
        f1 = @(x) cte * (sum(x.*x, 2)).^(gamma/2);
        f2 = @(x) cte * (sum((1-x).^2, 2)).^(gamma/2);
        z_ref = [10^gamma 10^gamma];
        % For this problem the exact solution is known:
        true_pareto_f1 = (0:1e-3:1)';
        true_pareto_f2 = (1-true_pareto_f1.^(1/gamma)).^gamma;
        dhv_theo = stk_dominatedhv ([true_pareto_f1 true_pareto_f2], z_ref);
end

% Upper constraints on objective functions
% (setting them to +Inf means 'no constraint')
z_constr = [Inf Inf];

% Setting upper constraints is actually equivalent to changing the ref point
% (from now on, any point that does not dominate z_ref is considered unfeasible)
z_ref = min (z_ref, z_constr);


%% Initial design of experiments

n0 = 10 * dim;  % Size of the initial design

% Construction of the initial design
x0 = stk_sampling_maximinlhs (n0, [], dom);

% Values of the function on the initial design
z0 = [stk_feval(f1, x0) stk_feval(f2, x0)];
z0.colnames = {'z1', 'z2'};


%% Initial Display

progress_monitor = figure ();
subplot (1, 2, 2);  stk_plot_boo_pareto (z0, z_ref);

if ~ isempty (true_pareto_f1)
    plot (true_pareto_f1, true_pareto_f2, '-');
end
drawnow


%% Specification of the model (Gaussian process prior)

model(1) = stk_model ('stk_materncov52_aniso', dim);
model(2) = stk_model ('stk_materncov52_aniso', dim);

% Estimate the parameters of the models based on initial design
model(1).param = stk_param_estim (model(1), x0, z0(:, 1));
model(2).param = stk_param_estim (model(2), x0, z0(:, 2));


%% Cross validation

[y_LOO1, res_LOO1] = stk_predict_leaveoneout (model(1), x0, z0(:, 1));
[y_LOO2, res_LOO2] = stk_predict_leaveoneout (model(2), x0, z0(:, 2));

figure;  set (gcf, 'Name', 'LOO cross-validation');
subplot(2, 2, 1);  stk_plot_predvsobs (z0(:, 1), y_LOO1);
subplot(2, 2, 2);  stk_plot_histnormres (res_LOO1.norm_res);
subplot(2, 2, 3);  stk_plot_predvsobs (z0(:, 2), y_LOO2);
subplot(2, 2, 4);  stk_plot_histnormres (res_LOO2.norm_res);  drawnow;


%% Sequential design of experiments
%
% Here, evaluations points are chosen sequentially using the Expected
% Hypervolume Improvement (EHVI) criterion, starting from the initial
% design defined above.
%
% The algorithm stops when the maximum number of iterations is reached.

aux = [];
EHVI_seq = [];
DHV_seq = [];

x = x0;
z = z0;

optim_options = struct ();
optim_options.method = 'SMC';          % See help stk_optim_ehvi for details
optim_options.population_size = [];    % empty means: use the default
optim_options.batch_size = batchSize;

i = 1;
while i <= NIter
    
    [x_new, ehvi_opt, aux] = stk_optim_ehvi ...
        (optim_options, dom, model, horzcat (x, z), z_ref, aux);
    
    % Evaluation of all the batch at the same time
    z_new = stk_feval ({f1 f2}, x_new);
    
    % Add the new evaluations to the data set
    x = vertcat (x, x_new);
    z = vertcat (z, z_new);

    % Extract non-dominated solutions
    idx_pareto = stk_paretofind_constrained (z, z_ref);
    x_pareto = x(idx_pareto, :);
    z_pareto = z(idx_pareto, :);
    
    % Compute dominated hypervolume
    DHV_seq = [DHV_seq stk_dominatedhv(z_pareto, z_ref)];
    
    i = i + batchSize;
    
    % Progress monitor
    figure (progress_monitor);
    set (gcf, 'Name', sprintf ('Progress at n=%d+%d=%d', n0, i, n0 + i));
    
    % Progress monitor, right panel: objective space
    subplot(1, 2, 2);  stk_plot_boo_pareto (z, z_ref, batchSize);
    stk_title ('objective space');
    if ~ isempty (true_pareto_f1)
        plot (true_pareto_f1, true_pareto_f2, '-');
    end
    
    % Progress monitor, left panel: progress
    subplot (1, 2, 1);
    stk_plot_moo_progress (DHV_seq, dhv_theo);  drawnow
    
    
    % Pause, if needed
    if mod (i - 1, NPause) == 0,  pause ();  end
    
    % Estimation of parameters if needed
    if mod(i, NEstimParam) == 0
        model(1).param = stk_param_estim (model(1), x, z(:,1));
        model(2).param = stk_param_estim (model(2), x, z(:,2));
    end
end


%#ok<*AGROW>
